# Copyright (C) 2006-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import collections
import logging
import threading
import time
from agent.lib.constants import (MEMBERSHIP_STATE_UNOWNED,
                                 MEMBERSHIP_STATE_OWNED, SLEEP_ON_ERR)
from agent.lib.settings import get_membership_state
from agent.target.cluster import is_clustered
from agent.lib.stats import sleep_for_statistic_interval, process_stats
from agent.target.stats import StatsCollector


def serve_forever():
    logging.info("STAT SERVER STARTED")
    queue = collections.deque()
    lock = threading.Lock()
    StatsCollector(queue, lock).start()
    while True:
        try:
            sleep_for_statistic_interval()
            if (is_clustered() or
                (get_membership_state() not in (MEMBERSHIP_STATE_UNOWNED,
                                               MEMBERSHIP_STATE_OWNED))):
                process_stats(queue, lock)
        except Exception, e:
            logging.exception("Error in stat process: %s", e)
            time.sleep(SLEEP_ON_ERR)


if __name__ == '__main__':
    serve_forever()
