# Copyright (C) 2006-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import cPickle
import logging
import os
import socket
import struct

from agent.lib.constants import LOCAL_SERVER_DB, NOTIFICATION_SERVER_ADDRESS
from agent.lib.db import read_item


def notify_manager(notification, data=None):
    url = get_manager_core_api_url()
    if not url:
        raise Exception("Cannot get manager URL from agent db.")

    if data is None:
        data = {}

    try:
        soc = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    except Exception, e:
        logging.error("Unable to get notification socket: %s", e)
        raise e

    try:
        msg = cPickle.dumps((notification, data))
        msglen = struct.pack(">L", len(msg))
        try:
            soc.connect(NOTIFICATION_SERVER_ADDRESS)
            soc.sendall(msglen + msg)
        except Exception, e:
            logging.error("Unable to send notification: %s", e)
            raise e
        if not notification.startswith('{STATS}'):
            logging.info('Notification sent: %s', notification)
    finally:
        soc.close()


def async_proc_prog(status):
    notification = '{ASYNC_PROC} progress PID %s' % (os.getpid(),)
    logging.info('async_proc_prog: notification %s status %s', notification, status)
    try:
        notify_manager(notification, status)
    except Exception, err:
        logging.warn('Error sending notification: %s status: %s error: %s',
                     notification, status, err)


def async_proc_cache(cache):
    notification = '{ASYNC_PROC} cache PID %s' % (os.getpid(),)
    try:
        notify_manager(notification, cache)
    except Exception, err:
        logging.warn('Error sending notification: %s cache: %s error: %s',
                     notification, cache, err)


def get_manager_core_api_url():
    """
    Return the URL supplied by the OVM Manager for making
    Core API calls.

    @param      None

    @return     URL string
    """
    return read_item(LOCAL_SERVER_DB, "manager_core_api_url")
