#!/usr/bin/env python
#
# Copyright (C) 2011-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for acquiring block device and disk information
"""

import st_utils
import st_tech

import os
import re
import traceback
import stat

from diskAccessControl	import *

####################################################################################################

__all__ = [	'DiskInfo_CurState_OBJ'
		,	'DiskInfo_OBJ'
		,	'get_blk_dev_list'
		,	'filter_blk_dev_list'
		,	'filter_blk_dev_list_emc'
		,	'populate_blk_dev_info_dict'
		,	'populate_blk_dev_info_dict_emc'
		,	'fill_blk_dev_info_dict____basic'
		,	'add_non_standard_blk_devs'
		,	'fill_blk_dev_info_dict____initial_disk_scan'
		,	'fill_blk_dev_info_dict____directed_disk_scan'
		,	'categorize_blk_dev_list' 
                ,       'fill_blk_dev_info_dict____multipath'
                ,       'fill_blk_dev_info_dict____emc']

####################################################################################################

class DiskInfo_CurState_OBJ:

	def __init__( self ):
		self.mWWID                    = None
		self.mWasAlreadyAccessible    = None

	def __repr__( self ):
		return "mWWID: %s mWasAlreadyAccessible: %s" % (
						self.mWWID
					,	self.mWasAlreadyAccessible )

class DiskInfo_OBJ:

	def __init__( self ):
		self.mWWID               = None
		self.mIsSCSIdevice       = None
		self.mCurState           = None # DiskInfo_CurState_OBJ
		self.mBlkDevInfo_LIST    = [] # [ BlkDevInfo_OBJ ]

	def __repr__( self ):
		return "mWWID: %s mIsSCSIdevice: %s mCurState: %s mBlkDevInfo_LIST: %s" % (
						self.mWWID
					,	self.mIsSCSIdevice
					,	self.mCurState
					,	self.mBlkDevInfo_LIST )

####################################################################################################
def get_blk_dev_list( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	blkdev_LIST = []

	for blkdev in os.listdir( '/dev' ):
		blkdev_path = "/dev/%s" % ( blkdev )
		try:
			mode = os.stat( blkdev_path )[stat.ST_MODE]
			if not stat.S_ISBLK( mode ):
				continue
			if os.path.islink( blkdev_path ):	# NOTE: S_ISLNK() can not be used in this case 
				continue
		except:
			continue
		if not os.path.exists( "/sys/block/%s" % ( blkdev ) ):
			continue
#		vi_errorAndLog_obj.vi_print_dbg( 0, blkdev )
		blkdev_LIST.append( blkdev )

	if os.path.exists( '/dev/cciss' ):
		for blkdev in os.listdir( '/dev/cciss' ):
			blkdev_path = "/dev/cciss/%s" % ( blkdev )
			try:
				mode = os.stat( blkdev_path )[stat.ST_MODE]
				if not stat.S_ISBLK( mode ):
					continue
				if os.path.islink( blkdev_path ):	# NOTE: S_ISLNK() can not be used in this case 
					continue
			except:
				continue
			if not os.path.exists( "/sys/block/cciss!%s" % ( blkdev ) ):
				continue
#			vi_errorAndLog_obj.vi_print_dbg( 0, blkdev )
			blkdev_LIST.append( "cciss!%s" % ( blkdev ) )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, "Block Devices:" )
		for blkdev in blkdev_LIST:
			vi_errorAndLog_obj.vi_print_dbg( 0, "    %s" % ( blkdev ) )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )

	return blkdev_LIST

####################################################################################################
def filter_blk_dev_list( vi_errorAndLog_obj, blkdev_LIST, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo ):
	"""
	This routine
	"""
#
	filtered_blkdev_LIST = []

	for blkdev in blkdev_LIST:
		for mpConf_blkLstDevNode in mpConf_blacklistInfo.mDevNode_LIST:
#			vi_errorAndLog_obj.vi_print_dbg( 0, "%s ] %s" % ( blkdev, mpConf_blkLstDevNode.mDevNode ) )
			if mpConf_blkLstDevNode.mDevNode_cmpldRegEx.search( blkdev ):	# match() assumes '^' anchor in pattern
				blkdev = None
				break
		if not blkdev:
			continue
#		vi_errorAndLog_obj.vi_print_dbg( 0, "%s [ GOOD" % ( blkdev ) )
		filtered_blkdev_LIST.append( blkdev )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, "Filtered Block Devices:" )
		for blkdev in filtered_blkdev_LIST:
			vi_errorAndLog_obj.vi_print_dbg( 0, "    %s" % ( blkdev ) )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )

	return filtered_blkdev_LIST

####################################################################################################
def filter_blk_dev_list_emc( vi_errorAndLog_obj, blkdev_LIST, filter_List ):
        """
        This routine
        """
#
        filtered_blkdev_LIST = []

        for blkdev in blkdev_LIST:
		found = False
		for item in filter_List:
			if blkdev.startswith(item):
				found = True
				break

		if found:
                	filtered_blkdev_LIST.append( blkdev )

        return filtered_blkdev_LIST

####################################################################################################
def populate_blk_dev_info_dict( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST, mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST  ):
	"""
	This routine
	"""
#
	cciss_blkDev_cmpldRegEx	= re.compile( '^cciss!' )

	for blkdev in blkdev_LIST:

		blkDevInfo	= st_tech.get_empty_blkDevInfo( vi_errorAndLog_obj, blkdev )
		if not blkDevInfo:
			continue

		if not cciss_blkDev_cmpldRegEx.match( blkdev ):
			if st_tech.fill_blkDevInfo_scsi_device_basics( vi_errorAndLog_obj, blkDevInfo, False ):
				continue
			blkDevInfo.mIsSCSIdevice	= True
			blkDevInfo.mDevPath			= "/dev/%s" % ( blkdev )
		else:
			blkDevInfo.mIsSCSIdevice	= False
			blkDevInfo.mDevPath			= "/dev/cciss/%s" % ( blkdev[6:] )

			cciss_sg_inq	= st_tech.do_sg_inq( vi_errorAndLog_obj, blkDevInfo.mDevPath )

			if cciss_sg_inq.mSCSI_vendor:
				blkDevInfo.mVendor	= cciss_sg_inq.mSCSI_vendor
			else:
				blkDevInfo.mVendor	= 'HP'	# only known cciss vendor

			if cciss_sg_inq.mSCSI_model:
				blkDevInfo.mProduct	= cciss_sg_inq.mSCSI_model
			else:
				blkDevInfo.mProduct	= 'SMART ARRAY'	

			if cciss_sg_inq.mSCSI_rev:
				blkDevInfo.mRev		= cciss_sg_inq.mSCSI_rev
			else:
				blkDevInfo.mRev		= '0.0'

#			if cciss_sg_inq.mSCSI_vendor == 'HP':
#				blkDevInfo.mVendor	= 'HP'
#				blkDevInfo.mProduct	= 'SMART ARRAY'
#				blkDevInfo.mRev		= ''

		is_blacklisted	= False
		for mpConf_blkLstDev in mpConf_blacklistInfo.mDevice_LIST:
			if not mpConf_blkLstDev.mVendor_cmpldRegEx.match( blkDevInfo.mVendor ):
				continue
			if not mpConf_blkLstDev.mProduct_cmpldRegEx.match( blkDevInfo.mProduct ):
				continue
			is_blacklisted	= True
			break
		if is_blacklisted:
			continue

		for mpConf_devSpec in mpConf_devSpec_LIST:
#			vi_errorAndLog_obj.vi_print_dbg( 0, "vp: %s %s | %s %s" % ( blkDevInfo.mVendor, blkDevInfo.mProduct, mpConf_devSpec.mVendor, mpConf_devSpec.mProduct ) )
			if mpConf_devSpec.mVendor_cmpldRegEx:
				if not mpConf_devSpec.mVendor_cmpldRegEx.search( blkDevInfo.mVendor ):		# match() assumes '^' anchor in pattern
#				vi_errorAndLog_obj.vi_print_dbg( 0, "vp: %s %s | v -" % ( blkDevInfo.mVendor, blkDevInfo.mProduct ) )
			        	continue
			if mpConf_devSpec.mProduct_cmpldRegEx:
				if not mpConf_devSpec.mProduct_cmpldRegEx.search( blkDevInfo.mProduct ):	# match() assumes '^' anchor in pattern
#				vi_errorAndLog_obj.vi_print_dbg( 0, "vp: %s %s | p -" % ( blkDevInfo.mVendor, blkDevInfo.mProduct ) )
					continue
#			vi_errorAndLog_obj.vi_print_dbg( 0, "VP: %s %s | %s" % ( blkDevInfo.mVendor, blkDevInfo.mProduct, mpConf_devSpec ) )
			break
		else:
			mpConf_devSpec	= None

		callout = mpConf_default_callout

		if mpConf_devSpec:
#			vi_errorAndLog_obj.vi_print_dbg( 0, "Spec: %s %s | %s" % ( blkDevInfo.mVendor, blkDevInfo.mProduct, mpConf_devSpec ) )
			if mpConf_devSpec.mBlacklist:
				if re.search( mpConf_devSpec.mBlacklist, blkDevInfo.mProduct ):	# match() assumes '^' anchor in pattern
					continue
			if mpConf_devSpec.mCallout:
				callout = mpConf_devSpec.mCallout
			blkDevInfo.mHasExplicitMPconf	= mpConf_devSpec.mIsExplicit
			blkDevInfo.mIsWWIDgenerated		= mpConf_devSpec.mIsCalloutGenerated
#			vi_errorAndLog_obj.vi_print_dbg( 0, "EXPL: %s | %s %s | %s" % ( blkDevInfo.mHasExplicitMPconf, blkDevInfo.mVendor, blkDevInfo.mProduct, mpConf_devSpec ) )

		final_callout = re.sub( '%n', blkdev, callout )

#		vi_errorAndLog_obj.vi_print_dbg( 0, "Callout: [%s](%s) %s" % ( blkDevInfo.mVendor, blkDevInfo.mProduct, final_callout ) )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, final_callout )
		if len(stdout_LIST) > 0:
			wwid = stdout_LIST[0].strip()
			if len(wwid) < 1:
				continue
		else:
			continue

		is_blacklisted	= False
		for mpConf_blkLstWWID in mpConf_blacklistInfo.mWWID_LIST:
                        if (mpConf_blkLstWWID.mWWID_cmpldRegEx and
                            not mpConf_blkLstWWID.mWWID_cmpldRegEx.search( wwid )):	# match() assumes '^' anchor in pattern
                                continue
                        for mpConf_blkLstExWWID in mpConf_blacklistExceptionsInfo.mWWID_LIST:
                                if mpConf_blkLstExWWID.mWWID_cmpldRegEx.search( wwid ):
                                        break
                        else:
                                is_blacklisted	= True
			break
		if is_blacklisted:
			continue

		blkDevInfo.mWWID	= wwid

#		vi_errorAndLog_obj.vi_print_dbg( 0, 'B:', blkDevInfo )

		try:
			diskInfo = diskInfo_DICT[ wwid ]
			diskInfo.mBlkDevInfo_LIST.append( blkDevInfo )
		except KeyError:
			diskInfo                   = DiskInfo_OBJ()
			diskInfo.mWWID             = wwid
			diskInfo.mIsSCSIdevice     = blkDevInfo.mIsSCSIdevice
			diskInfo.mCurState         = DiskInfo_CurState_OBJ()
			diskInfo.mCurState.mWWID   = diskInfo.mWWID
			diskInfo.mBlkDevInfo_LIST  = [ blkDevInfo ]
			diskInfo_DICT[ wwid ]      = diskInfo

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		sorted_wwid_LIST = diskInfo_DICT.keys()
		sorted_wwid_LIST.sort()
		for wwid in sorted_wwid_LIST:
			diskInfo = diskInfo_DICT[ wwid ]
			for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
				vi_errorAndLog_obj.vi_print_dbg( 0, 'B:', blkDevInfo )
				vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_stop()

####################################################################################################
def populate_blk_dev_info_dict_emc( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST ):
	"""
	This routine
	"""
#

	if not os.path.exists("/dev/emcpower"):
	    return

	for blkdev in blkdev_LIST:

		blkDevInfo	= st_tech.get_empty_blkDevInfo( vi_errorAndLog_obj, blkdev )
		if not blkDevInfo:
			continue

		if st_tech.fill_blkDevInfo_scsi_device_basics( vi_errorAndLog_obj, blkDevInfo, False ):
			continue

		blkDevInfo.mIsSCSIdevice	= True
		blkDevInfo.mDevPath		= "/dev/%s" % ( blkdev )


		callout = "/sbin/scsi_id -g -u -s /block/%s" % blkdev

		stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, callout )
		if len(stdout_LIST) > 0:
			wwid = stdout_LIST[0].strip()
			if len(wwid) < 1:
				continue
		else:
			continue

		blkDevInfo.mWWID	= wwid

		try:
			diskInfo = diskInfo_DICT[ wwid ]
			diskInfo.mBlkDevInfo_LIST.append( blkDevInfo )
		except KeyError:
			diskInfo                   = DiskInfo_OBJ()
			diskInfo.mWWID             = wwid
			diskInfo.mIsSCSIdevice     = blkDevInfo.mIsSCSIdevice
			diskInfo.mCurState         = DiskInfo_CurState_OBJ()
			diskInfo.mCurState.mWWID   = diskInfo.mWWID
			diskInfo.mBlkDevInfo_LIST  = [ blkDevInfo ]
			diskInfo_DICT[ wwid ]      = diskInfo



####################################################################################################
def fill_blk_dev_info_dict____basic( vi_errorAndLog_obj, diskInfo_DICT, qla4xxx_state_info_DICT, openiscsi_session_info_DICT ):
	"""
	This routine does everything that does not require accessing the disk itself
	"""
#
	wwidsToDelete_LIST	= []
	for wwid, diskInfo in diskInfo_DICT.iteritems():
		blkDevIdxsToDelete_LIST	= []	# Be more efficient than always making a slice copy

		if diskInfo.mIsSCSIdevice:
			for idx, blkDevInfo in enumerate(diskInfo.mBlkDevInfo_LIST):
				if st_tech.fill_blkDevInfo_scsi_device( vi_errorAndLog_obj, blkDevInfo, False ):
					blkDevIdxsToDelete_LIST.append( idx )
					continue
				blkDevInfo.mOperations	= None

		else:	# can only be CCISS for now
			for idx, blkDevInfo in enumerate(diskInfo.mBlkDevInfo_LIST):

				if st_tech.fill_blkDevInfo_CCISS_device( vi_errorAndLog_obj, blkDevInfo, False ):

					vi_errorAndLog_obj.vi_print_err( "Excpected CCISS disk was not : %s" % ( blkDevInfo ) )

					blkDevIdxsToDelete_LIST.append( idx )
					continue

				blkDevInfo.mOperations	= None

		if len( blkDevIdxsToDelete_LIST ) < 1:
			continue

		if len( blkDevIdxsToDelete_LIST ) == len( diskInfo.mBlkDevInfo_LIST ):
			wwidsToDelete_LIST.append( wwid )
			continue

		for idx in reversed(blkDevIdxsToDelete_LIST):
			del( diskInfo.mBlkDevInfo_LIST[ idx ] )

	for wwid in wwidsToDelete_LIST:
		del( diskInfo_DICT[wwid] )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		sorted_wwid_LIST = diskInfo_DICT.keys()
		sorted_wwid_LIST.sort()
		for wwid in sorted_wwid_LIST:
			diskInfo = diskInfo_DICT[ wwid ]
			for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
				vi_errorAndLog_obj.vi_print_dbg( 0, blkDevInfo )
				vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )

####################################################################################################
def add_non_standard_blk_devs( vi_errorAndLog_obj, diskInfo_DICT ):
	"""
	This routine
	"""
#
	SPECIAL_PARTITION_DISK_PATH_SPECIFIER_PATH		= "/tmp/evm_volgroup_dev"
	SPECIAL_PARTITION_DISK_BLK_DEV_SPECIFIER_PATH	= "/tmp/evm_rootname"

	#
	# Handle SSE / EVM special boot disk partition storage quasi disk
	#
	while os.path.exists( SPECIAL_PARTITION_DISK_PATH_SPECIFIER_PATH ):

		partitionDiskPath, read_failed = st_utils.vi_read_text_value( vi_errorAndLog_obj, SPECIAL_PARTITION_DISK_PATH_SPECIFIER_PATH )
		if read_failed:
			vi_errorAndLog_obj.vi_print_err( "specialPartitionDiskPathSpecifierPath %s exists but is unreadable" % ( SPECIAL_PARTITION_DISK_PATH_SPECIFIER_PATH ) )
			break

		partitionDiskBlkDev, read_failed = st_utils.vi_read_text_value( vi_errorAndLog_obj, SPECIAL_PARTITION_DISK_BLK_DEV_SPECIFIER_PATH )
		if read_failed:
			vi_errorAndLog_obj.vi_print_err( "specialPartitionDiskBlkDevSpecifier %s is unreadable" % ( SPECIAL_PARTITION_DISK_BLK_DEV_SPECIFIER_PATH ) )
			break

		if not os.path.exists( partitionDiskPath ):
			vi_errorAndLog_obj.vi_print_err( "partitionDiskPath %s [%s] does not exist" % ( partitionDiskPath, SPECIAL_PARTITION_DISK_PATH_SPECIFIER_PATH ) )
			break

		if not partitionDiskPath.startswith( '/dev/mapper/' ):
			vi_errorAndLog_obj.vi_print_err( "partitionDiskPath %s [%s] has unexpected format" % ( partitionDiskPath, SPECIAL_PARTITION_DISK_PATH_SPECIFIER_PATH ) )
			break

		partitionDiskWWID	= partitionDiskPath[12:]

		for wwid, diskInfo in diskInfo_DICT.iteritems():
			for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
				if blkDevInfo.mBlkDev == partitionDiskBlkDev:
					break
			else:
				continue
			# Found : now usurp for the partitionDisk
			diskInfo.mWWID						= partitionDiskWWID
			diskInfo.mCurState.mWWID			= partitionDiskWWID
			diskInfo.mIsSCSIdevice				= False
			for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
				blkDevInfo.mWWID				= partitionDiskWWID
			diskInfo_DICT[ partitionDiskWWID ]	= diskInfo
			del( diskInfo_DICT[ wwid ] )
			break
		else:
			vi_errorAndLog_obj.vi_print_err( "partitionDisk %s [%s] %s was not found" % ( partitionDiskPath, partitionDiskBlkDev, partitionDiskWWID ) )

		break

	aoe_disk_LIST = st_tech.find_aoe_disks( vi_errorAndLog_obj, False )

	for aoe_disk in aoe_disk_LIST:

		blkDevInfo	= st_tech.get_empty_blkDevInfo( vi_errorAndLog_obj, aoe_disk.mBlkDev )
		if not blkDevInfo:
			continue

		blkDevInfo.mWWID	= aoe_disk.mWWID

		if st_tech.fill_blkDevInfo_AOE_device( vi_errorAndLog_obj, blkDevInfo, aoe_disk ):
			continue

		blkDevInfo.mOperations	= None

		diskInfo = diskInfo_DICT.get( blkDevInfo.mWWID, None )
		if diskInfo:
			curBlkDevInfo = diskInfo.mBlkDevInfo_LIST[ 0 ]
			diskInfo.mBlkDevInfo_LIST.append( blkDevInfo )
#			vi_errorAndLog_obj.vi_croak( "aoe disk wwid collision %s vs %s !?!" % ( blkDevInfo.mWWID, blkDevInfo.mBlkDev, curBlkDevInfo.mBlkDev ) )
			vi_errorAndLog_obj.vi_print_err( "aoe disk wwid collision %s vs %s !?!" % ( blkDevInfo.mWWID, blkDevInfo.mBlkDev, curBlkDevInfo.mBlkDev ) )
		else:
			diskInfo                          = DiskInfo_OBJ()
			diskInfo.mWWID                    = blkDevInfo.mWWID
			diskInfo.mIsSCSIdevice            = False
			diskInfo.mCurState                = DiskInfo_CurState_OBJ()
			diskInfo.mCurState.mWWID          = diskInfo.mWWID
			diskInfo.mBlkDevInfo_LIST         = [ blkDevInfo ]
			diskInfo_DICT[ blkDevInfo.mWWID ] = diskInfo

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		sorted_wwid_LIST = diskInfo_DICT.keys()
		sorted_wwid_LIST.sort()
		for wwid in sorted_wwid_LIST:
			vi_errorAndLog_obj.diskInfo = diskInfo_DICT[ wwid ]
			for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
				vi_errorAndLog_obj.vi_print_dbg( 0, blkDevInfo )
				vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )

####################################################################################################
def fill_blk_dev_info_dict____initial_disk_scan( vi_errorAndLog_obj, diskInfo_DICT, skipLVM ):
	"""
	This routine

	NOTE:	When performing the initial disk scan we do NOT know which
			disks are independent and which ones form a set of PVs for
			a VG.  So always get PV information just in case, but also 
			capture as much of the other info as happens to be available
			along the way (as the single PV per VG is the norm).
	"""
#
	pvInfo_uuid_DICT = {}
	vgInfo_uuid_DICT = {}
	

	sorted_wwid_LIST = diskInfo_DICT.keys() # Use sorted keys as people tend to use sequential disks for forming multi-disk VGs
	sorted_wwid_LIST.sort()
	wwid_LIST = []
	wwid_LIST_LIST = []
	for wwid in sorted_wwid_LIST:
		wwid_LIST.append( wwid )
		if len(wwid_LIST) > global_numOfDisksSetupAtOnce:
			wwid_LIST_LIST.append( wwid_LIST )
			wwid_LIST = []
	if len(wwid_LIST) > 0:
		wwid_LIST_LIST.append( wwid_LIST )

#	for wwid_LIST in wwid_LIST_LIST:

#		for wwid in wwid_LIST:
#			setup_disk_for_access( vi_errorAndLog_obj, wwid, diskInfo_DICT )

#		multipathInfo_DICT = st_tech.get_multipath_info( vi_errorAndLog_obj, False )

#		if not skipLVM:
#			pvInfo_uuid_DICT, vgInfo_uuid_DICT = st_tech.add_lvm_info( vi_errorAndLog_obj, True, pvInfo_uuid_DICT, vgInfo_uuid_DICT )

#		for wwid in wwid_LIST: # Set mp state (UP vs DOWN) info

#			diskInfo = diskInfo_DICT[ wwid ]

#			for blkDevInfo in diskInfo.mBlkDevInfo_LIST[:]:
#				try:
#					blkDev_DICT                          = multipathInfo_DICT[ wwid ]
#					mpBlkDevInfo                         = blkDev_DICT.get(blkDevInfo.mBlkDev, None)
#					if mpBlkDevInfo == None:
#						# Race condition. The devmon calls the same script to delete from the cache entries about
						# this block device (and multipath -ll already has it deleted), but we are still using old
						# cache data
#						diskInfo.mBlkDevInfo_LIST.remove(blkDevInfo)
#						continue
#					blkDevInfo.mCurState.mPathDMmapState = mpBlkDevInfo.mPathDMmapState
#					blkDevInfo.mCurState.mCheckerState   = mpBlkDevInfo.mCheckerState
#					vi_errorAndLog_obj.vi_print_dbg( 0, blkDevInfo.mBlkDev, blkDevInfo.mCurState.mPathDMmapState, blkDevInfo.mCurState.mCheckerState )
#				except KeyError:
#					if diskInfo.mIsSCSIdevice:
						# (unless multipath died) The HWtable inside multipath blacklisted all of the found blkDevs for this disk
#						vi_errorAndLog_obj.vi_print_dbg( 0, "Assumed blacklisted: %s", diskInfo.mBlkDevInfo_LIST )
#						del( diskInfo_DICT[ wwid ] )
#						break

#		if len(diskInfo.mBlkDevInfo_LIST) == 0:
#			del (diskInfo_DICT[ wwid ] )

#		del( multipathInfo_DICT )

		for wwid in wwid_LIST:
			done_accessing_disk( vi_errorAndLog_obj, wwid, diskInfo_DICT )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		sorted_wwid_LIST = diskInfo_DICT.keys()
		sorted_wwid_LIST.sort()
		for wwid in sorted_wwid_LIST:
			diskInfo = diskInfo_DICT[ wwid ]
			for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
				vi_errorAndLog_obj.vi_print_dbg( 0, blkDevInfo )
				vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )

	return ( pvInfo_uuid_DICT, vgInfo_uuid_DICT )

####################################################################################################
def fill_blk_dev_info_dict____directed_disk_scan( vi_errorAndLog_obj, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine

	NOTE:	Go through the missing VGs and setup all of the disks
			that claimed to part of the PV.  NOTE:  If a disk is
			missing this still will not work.
	"""
#
	wwid_LIST = []
	wwid_LIST_LIST = []
	for vg_name, pvInfo_LIST in missingVGs_DICT.iteritems():
		if ( len(wwid_LIST) + len(pvInfo_LIST) ) > global_numOfDisksSetupAtOnce:
			if len(wwid_LIST) > 0:
				wwid_LIST.sort()
				wwid_LIST_LIST.append( wwid_LIST )
				wwid_LIST = []
		for pv_info in pvInfo_LIST:
			wwid_LIST.append( pv_info.mWWID )
	if len(wwid_LIST) > 0:
		wwid_LIST.sort()
		wwid_LIST_LIST.append( wwid_LIST )

	for wwid_LIST in wwid_LIST_LIST:

		for wwid in wwid_LIST:
			setup_disk_for_access( vi_errorAndLog_obj, wwid, diskInfo_DICT )

		pvInfo_uuid_DICT, vgInfo_uuid_DICT = st_tech.add_lvm_info( vi_errorAndLog_obj, False, pvInfo_uuid_DICT, vgInfo_uuid_DICT )

		for wwid in wwid_LIST:
			done_accessing_disk( vi_errorAndLog_obj, wwid, diskInfo_DICT )

	return ( pvInfo_uuid_DICT, vgInfo_uuid_DICT )

####################################################################################################
def categorize_blk_dev_list( vi_errorAndLog_obj, item_LIST ):
	"""
	This routine
	"""
#
	aoe_cmpldRegEx   = re.compile( '^etherd!' )

	blkdev_LIST             = []
	nonStandard_blkdev_LIST = []
	removed_blkdev_DICT     = {}

	for blkdev in item_LIST:
		if not os.path.exists( "/sys/block/%s" % ( blkdev ) ):
			removed_blkdev_DICT[ blkdev ] = True
			continue
		if aoe_cmpldRegEx.match( blkdev ):
			nonStandard_blkdev_LIST.append( blkdev )
			continue
		blkdev_LIST.append( blkdev )

	return ( blkdev_LIST, nonStandard_blkdev_LIST, removed_blkdev_DICT )
####################################################################################################
def fill_blk_dev_info_dict____multipath( vi_errorAndLog_obj, diskInfo_DICT):
	"""
	
	"""
#
	

	sorted_wwid_LIST = diskInfo_DICT.keys() 
	sorted_wwid_LIST.sort()
	wwid_LIST = []
	wwid_LIST_LIST = []
	for wwid in sorted_wwid_LIST:
		wwid_LIST.append( wwid )
		if len(wwid_LIST) > global_numOfDisksSetupAtOnce:
			wwid_LIST_LIST.append( wwid_LIST )
			wwid_LIST = []
	if len(wwid_LIST) > 0:
		wwid_LIST_LIST.append( wwid_LIST )

 	for wwid_LIST in wwid_LIST_LIST:

 		for wwid in wwid_LIST:
 			setup_disk_for_access( vi_errorAndLog_obj, wwid, diskInfo_DICT )

 		multipathInfo_DICT = st_tech.get_multipath_info( vi_errorAndLog_obj, False )

 		for wwid in wwid_LIST: # Set mp state (UP vs DOWN) info

 			diskInfo = diskInfo_DICT[ wwid ]

 			for blkDevInfo in diskInfo.mBlkDevInfo_LIST[:]:
 				try:
 					blkDev_DICT                          = multipathInfo_DICT[ wwid ]
 					mpBlkDevInfo                         = blkDev_DICT.get(blkDevInfo.mBlkDev, None)
 					if mpBlkDevInfo == None:
 						# Race condition. The devmon calls the same script to delete from the cache entries about
						# this block device (and multipath -ll already has it deleted), but we are still using old
						# cache data
 						diskInfo.mBlkDevInfo_LIST.remove(blkDevInfo)
 						continue
 					blkDevInfo.mCurState.mPathDMmapState = mpBlkDevInfo.mPathDMmapState
 					blkDevInfo.mCurState.mCheckerState   = mpBlkDevInfo.mCheckerState
                                        blkDevInfo.mDevMapNum                =  "/dev/dm-%s" % mpBlkDevInfo.mDevMapNum
 					# vi_errorAndLog_obj.vi_print_dbg( 0, blkDevInfo.mBlkDev, blkDevInfo.mCurState.mPathDMmapState, blkDevInfo.mCurState.mCheckerState )
 				except KeyError:
 					if diskInfo.mIsSCSIdevice:
						# (unless multipath died) The HWtable inside multipath blacklisted all of the found blkDevs for this disk
 						# vi_errorAndLog_obj.vi_print_dbg( 0, "Assumed blacklisted: %s", diskInfo.mBlkDevInfo_LIST )
 						del( diskInfo_DICT[ wwid ] )
 						break


 		del( multipathInfo_DICT )

		for wwid in wwid_LIST:
			done_accessing_disk( vi_errorAndLog_obj, wwid, diskInfo_DICT )

	
####################################################################################################
def fill_blk_dev_info_dict____emc( vi_errorAndLog_obj, diskInfo_DICT):
        """

        """
#

	for wwid, diskInfo in diskInfo_DICT.items():
		for blkDevInfo in diskInfo.mBlkDevInfo_LIST[:]:
			blkDevInfo.mDevMapNum = "unavailable"
			stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, "powermt display dev=%s" % blkDevInfo.mBlkDev )
			del( stderr_LIST )

			power_lines_LIST = map( str.strip, stdout_LIST )
			for line in power_lines_LIST:
				if line.find('Pseudo name=') >= 0:
					blkDevInfo.mDevMapNum = "/dev/%s" % line.split('=')[1].strip()
				items = line.split()
				for idx in range(len(items)):
					if items[idx] == blkDevInfo.mBlkDev: 
						blkDevInfo.mCurState.mCheckerState   = items[idx + 3]
						blkDevInfo.mCurState.mPathDMmapState = items[idx + 4]
			
			del( stdout_LIST )

		if blkDevInfo.mDevMapNum == "unavailable":
			del diskInfo_DICT[ wwid ]			




####################################################################################################
# END
