#!/usr/bin/env python
#
# Copyright (C) 2011-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Directives for managing the storage discovery process
"""

import st_utils
import st_tech

import os
import time
import traceback

from diskInfoCacheUtils	import *
from blkDevInfoUtils		import *

####################################################################################################

__all__ = [	'no_cache_change'
		,	'delete_entire_cache'
		,	'add_all_block_data'
		,	'add_specific_block_data'
		,	'remove_specific_block_data'
		,	'update_cached_block_data'
		,	'ensure_specific_disk_and_lvm_data' ]

####################################################################################################
def figure_out_missingVGs( pvInfo_uuid_DICT, vgInfo_uuid_DICT ):
	"""
	This routine
	"""
#
	vgNameExists_DICT = {}
	for vg_uuid in vgInfo_uuid_DICT:
		vg_info = vgInfo_uuid_DICT[ vg_uuid ]
		vgNameExists_DICT[ vg_info.mName ] = True

	missingVGs_DICT = {}
	for pv_uuid in pvInfo_uuid_DICT:
		pv_info = pvInfo_uuid_DICT[ pv_uuid ]
		if not pv_info.mVGname:
			continue
		if vgNameExists_DICT.has_key( pv_info.mVGname ):
			continue
		pvInfo_LIST = missingVGs_DICT.get( pv_info.mVGname, None )
		if not pvInfo_LIST:
			missingVGs_DICT[ pv_info.mVGname ] = [ pv_info ]
		else:
			pvInfo_LIST.append( pv_info )

	return missingVGs_DICT

####################################################################################################
def no_cache_change( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "no_cache_change" )
	startTime = time.time()

	diskInfo_DICT = pvInfo_uuid_DICT = vgInfo_uuid_DICT = missingVGs_DICT = {}
	try:
		diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT = read_blk_and_disk_cache( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	cacheEndTime = time.time()

	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs (%0.2f)" % ( ( endTime - startTime ),  ( cacheEndTime - startTime ) ) )

####################################################################################################
def delete_entire_cache( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "delete_entire_cache" )
	startTime = time.time()

	delete_blk_and_disk_cache_cache( vi_errorAndLog_obj )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs" % ( ( endTime - startTime ) ) )

####################################################################################################
def add_all_block_data( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "add_all_block_data" )

	startTime = time.time()

	try:
		qla4xxx_state_info_DICT = st_tech.get_qla4xxx_info_dict( vi_errorAndLog_obj, False )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		openiscsi_session_info_DICT = st_tech.get_openiscsi_session_info_dict( vi_errorAndLog_obj, False )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST = \
			st_tech.get_multipath_conf( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		blkdev_LIST = get_blk_dev_list( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	prepEndTime = time.time()


	diskInfo_DICT = {}
	if not os.path.exists("/dev/emcpower"):
		try:
			blkdev_LIST = filter_blk_dev_list( vi_errorAndLog_obj, blkdev_LIST, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		try:
			populate_blk_dev_info_dict( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST, mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	else:
		try:
                	find_List = ["sd"]
                	blkdev_LIST = filter_blk_dev_list_emc( vi_errorAndLog_obj, blkdev_LIST, find_List )
                	populate_blk_dev_info_dict_emc( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST )
        	except:
#               	vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
                	vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		fill_blk_dev_info_dict____basic( vi_errorAndLog_obj, diskInfo_DICT, qla4xxx_state_info_DICT, openiscsi_session_info_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		add_non_standard_blk_devs( vi_errorAndLog_obj, diskInfo_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	blkDevEndTime = time.time()

        if options_OBJ.infoDumpType == 'disk_XML':
		if not os.path.exists("/dev/emcpower"):
                	try:
		        	fill_blk_dev_info_dict____multipath( vi_errorAndLog_obj, diskInfo_DICT )
			except:
#				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		        	vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		else:
                	try:
		        	fill_blk_dev_info_dict____emc( vi_errorAndLog_obj, diskInfo_DICT )
			except:
#				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		        	vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )


        if options_OBJ.infoDumpType != 'disk_XML' and options_OBJ.infoDumpType != 'io_host_XML':
	        try:
		        write_blk_and_disk_cache( vi_errorAndLog_obj, diskInfo_DICT )
	        except:
#		        vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		        vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	        cacheEndTime = time.time()

	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, {}, {}, {} )

	endTime = time.time()

#	vi_errorAndLog_obj.vi_print_dbg( 1 )
#	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs [%0.2f] %0.2f (%0.2f)" % ( ( endTime - startTime ), ( prepEndTime - startTime ), ( blkDevEndTime - startTime ), ( cacheEndTime - startTime ) ) )

####################################################################################################
def get_block_data( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#

	startTime = time.time()

	try:
		qla4xxx_state_info_DICT = st_tech.get_qla4xxx_info_dict( vi_errorAndLog_obj, False )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		openiscsi_session_info_DICT = st_tech.get_openiscsi_session_info_dict( vi_errorAndLog_obj, False )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST = \
			st_tech.get_multipath_conf( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		blkdev_LIST = get_blk_dev_list( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	prepEndTime = time.time()

	try:
		blkdev_LIST = filter_blk_dev_list( vi_errorAndLog_obj, blkdev_LIST, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		diskInfo_DICT = {}
		populate_blk_dev_info_dict( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST, mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )



	try:
		fill_blk_dev_info_dict____basic( vi_errorAndLog_obj, diskInfo_DICT, qla4xxx_state_info_DICT, openiscsi_session_info_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		add_non_standard_blk_devs( vi_errorAndLog_obj, diskInfo_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	blkDevEndTime = time.time()

        
	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, {}, {}, {} )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs [%0.2f] %0.2f (%0.2f)" % ( ( endTime - startTime ), ( prepEndTime - startTime ), ( blkDevEndTime - startTime ), ( cacheEndTime - startTime ) ) )

####################################################################################################
def add_specific_block_data( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "add_specific_block_data" )
	startTime = time.time()

	try:
		blkdev_LIST, nonStandard_blkdev_LIST, removed_blkdev_DICT = categorize_blk_dev_list( vi_errorAndLog_obj, item_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	diskInfo_DICT = {}

	if len( blkdev_LIST ) > 0:
		try:
			qla4xxx_state_info_DICT = st_tech.get_qla4xxx_info_dict( vi_errorAndLog_obj, False )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

		try:
			openiscsi_session_info_DICT = st_tech.get_openiscsi_session_info_dict( vi_errorAndLog_obj, False )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

		try:
			mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST = \
				st_tech.get_multipath_conf( vi_errorAndLog_obj )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

		try:
			blkdev_LIST = filter_blk_dev_list( vi_errorAndLog_obj, blkdev_LIST, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

		try:
			diskInfo_DICT = {}
			populate_blk_dev_info_dict( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST, mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

		try:
			fill_blk_dev_info_dict____basic( vi_errorAndLog_obj, diskInfo_DICT, qla4xxx_state_info_DICT, openiscsi_session_info_DICT )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	if len( nonStandard_blkdev_LIST ) > 0:
		try:
			add_non_standard_blk_devs( vi_errorAndLog_obj, diskInfo_DICT )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	blkDevEndTime = time.time()

	try:
		write_blk_and_disk_cache( vi_errorAndLog_obj, diskInfo_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

#	if len( removed_blkdev_DICT ) > 0:
#		try:
#			remove_blk_devs_from_cache( vi_errorAndLog_obj, removed_blkdev_DICT )
#		except:
##			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
#			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	cacheEndTime = time.time()

	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, {}, {}, {} )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs %0.2f (%0.2f)" % ( ( endTime - startTime ), ( blkDevEndTime - startTime ), ( cacheEndTime - startTime ) ) )

####################################################################################################
def remove_specific_block_data( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "remove_specific_block_data" )
	startTime = time.time()

	try:
		blkdev_LIST, nonStandard_blkdev_LIST, removed_blkdev_DICT = categorize_blk_dev_list( vi_errorAndLog_obj, item_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	for blkdev in blkdev_LIST:
		removed_blkdev_DICT[ blkdev ] = True

	for blkdev in nonStandard_blkdev_LIST:
		removed_blkdev_DICT[ blkdev ] = True

	blkDevEndTime = time.time()

	if len( removed_blkdev_DICT ) > 0:
		try:
			remove_blk_devs_from_cache( vi_errorAndLog_obj, removed_blkdev_DICT )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	cacheEndTime = time.time()

	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, {}, {}, {}, {} )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs %0.2f (%0.2f)" % ( ( endTime - startTime ), ( blkDevEndTime - startTime ), ( cacheEndTime - startTime ) ) )

####################################################################################################
def update_cached_block_data( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "update_cached_block_data" )

	startTime = time.time()

	try:
		cached_diskInfo_DICT = read_all_blk_cache( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		qla4xxx_state_info_DICT = st_tech.get_qla4xxx_info_dict( vi_errorAndLog_obj, False )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		openiscsi_session_info_DICT = st_tech.get_openiscsi_session_info_dict( vi_errorAndLog_obj, False )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST = \
			st_tech.get_multipath_conf( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		blkdev_LIST = []
		for wwid in cached_diskInfo_DICT:
			diskInfo = cached_diskInfo_DICT[ wwid ]
			if diskInfo.mIsSCSIdevice:
				for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
					if os.path.exists( "/sys/block/%s" % ( blkDevInfo.mBlkDev ) ):
						blkdev_LIST.append( blkDevInfo.mBlkDev )
					else:
						vi_errorAndLog_obj.vi_print_dbg( 4, "BlkDev no longer exists :: %s" % ( blkDevInfo.mBlkDev ) )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	prepEndTime = time.time()

	try:
		diskInfo_DICT = {}
		populate_blk_dev_info_dict( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST, mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		fill_blk_dev_info_dict____basic( vi_errorAndLog_obj, diskInfo_DICT, qla4xxx_state_info_DICT, openiscsi_session_info_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	try:
		non_standard_diskInfo_DICT = {}
		add_non_standard_blk_devs( vi_errorAndLog_obj, non_standard_diskInfo_DICT )

		new_wwids_LIST = []
		for wwid in non_standard_diskInfo_DICT:
			if cached_diskInfo_DICT.has_key( wwid ):
				continue
			new_wwids_LIST.append( wwid )
		for wwid in new_wwids_LIST:
			del( non_standard_diskInfo_DICT[ wwid ] )

		diskInfo_DICT.update( non_standard_diskInfo_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	blkDevEndTime = time.time()

	try:
		write_blk_and_disk_cache( vi_errorAndLog_obj, diskInfo_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	cacheEndTime = time.time()

	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, {}, {}, {} )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs [%0.2f] %0.2f (%0.2f)" % ( ( endTime - startTime ), ( prepEndTime - startTime ), ( blkDevEndTime - startTime ), ( cacheEndTime - startTime ) ) )

####################################################################################################
def ensure_specific_disk_and_lvm_data( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "ensure_specific_disk_and_lvm_data" )
	startTime = time.time()

	if 0:	# UNTIL_CACHE_IS_ALWAYS_VALID
		delete_entire_cache( vi_errorAndLog_obj, options_OBJ, args_LIST, [], no_output )
		add_all_block_data( vi_errorAndLog_obj, options_OBJ, args_LIST, [], no_output )

	prepEndTime = time.time()

	try:
		diskInfo_DICT = read_specific_blk_cache( vi_errorAndLog_obj, item_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	blkDevEndTime = time.time()

	pvInfo_uuid_DICT = vgInfo_uuid_DICT = {}
	try:
		pvInfo_uuid_DICT, vgInfo_uuid_DICT = fill_blk_dev_info_dict____initial_disk_scan( vi_errorAndLog_obj, diskInfo_DICT, options_OBJ.skipLVM )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

	if not options_OBJ.skipLVM:
		missingVGs_DICT = figure_out_missingVGs( pvInfo_uuid_DICT, vgInfo_uuid_DICT )
		if len( missingVGs_DICT ):
			try:
				pvInfo_uuid_DICT, vgInfo_uuid_DICT = fill_blk_dev_info_dict____directed_disk_scan( vi_errorAndLog_obj, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT )
				missingVGs_DICT = figure_out_missingVGs( pvInfo_uuid_DICT, vgInfo_uuid_DICT )
			except:
#				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

		try:
			merge_and_write_lvm_cache( vi_errorAndLog_obj, vgInfo_uuid_DICT, missingVGs_DICT, item_LIST )
		except:
#			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
	else:
		missingVGs_DICT = {}

	endTime = time.time()

	cacheEndTime = time.time()

	infoDumpTypeFcn( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT )

	endTime = time.time()

	vi_errorAndLog_obj.vi_print_dbg( 1 )
	vi_errorAndLog_obj.vi_print_dbg( 1, "Total time: %0.2f secs [%0.2f] %0.2f (%0.2f)" % ( ( endTime - startTime ), ( prepEndTime - startTime ), ( blkDevEndTime - startTime ), ( cacheEndTime - startTime ) ) )

####################################################################################################
# END
