#!/usr/bin/env python
#
# Copyright (C) 2011-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Dump type options for managing the storage discover process
"""

import st_utils
import st_tech
import traceback

from blkDevInfoUtils		import *
from diskInfoCacheUtils	        import *
from storageXML			import *
from vdiskXML			import *

####################################################################################################

__all__ = [	'no_output'
		,	'disk_list'
		,	'disk_blkdev_list'
		,	'disk_blkdev_diff_list'
		,	'lvm_list'
		,	'block_data_XML'
		,	'disk_and_lvm_data_XML'
		,	'scan_vdisk_data_XML' 
		,	'disk_XML' 
		,	'io_host_XML' ]

####################################################################################################
def generate_disk_blkdev_differences_LIST( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	try:
		mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST = \
			st_tech.get_multipath_conf( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		return {}

	try:
		blkdev_LIST = get_blk_dev_list( vi_errorAndLog_obj )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		return {}

	try:
		blkdev_LIST = filter_blk_dev_list( vi_errorAndLog_obj, blkdev_LIST, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		return {}

	try:
		diskInfo_DICT = {}
		populate_blk_dev_info_dict( vi_errorAndLog_obj, diskInfo_DICT, blkdev_LIST, mpConf_default_callout, mpConf_blacklistInfo, mpConf_blacklistExceptionsInfo, mpConf_devSpec_LIST  )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		return {}

	try:
		add_non_standard_blk_devs( vi_errorAndLog_obj, diskInfo_DICT )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )
		return {}

	cached_diskInfo_DICT = read_all_blk_cache( vi_errorAndLog_obj )

	missing_diskInfo_DICT = {} # Exists but missing from the cache
	invalid_diskInfo_DICT = {} # Does not exist but still in cache

	cached_diskInfo_wwid_LIST = cached_diskInfo_DICT.keys()
	while len( cached_diskInfo_wwid_LIST ) > 0:
		try:
			while len( cached_diskInfo_wwid_LIST ) > 0:
				wwid = cached_diskInfo_wwid_LIST.pop()
				cached_diskInfo = cached_diskInfo_DICT[ wwid ]
				diskInfo = diskInfo_DICT.pop( wwid ) # Will raise() when does not exist but still in cache

				missing_blkDevInfo_LIST = diskInfo.mBlkDevInfo_LIST        # Exists but missing from the cache
				invalid_blkDevInfo_LIST = cached_diskInfo.mBlkDevInfo_LIST # Does not exist but still in cache

				invalid_blkDevInfo_idx = 0
				while invalid_blkDevInfo_idx < len( invalid_blkDevInfo_LIST ):
					invalid_blkDevInfo = invalid_blkDevInfo_LIST[ invalid_blkDevInfo_idx ]
					missing_blkDevInfo_idx = 0
					for missing_blkDevInfo in missing_blkDevInfo_LIST:
						if missing_blkDevInfo.mBlkDev == invalid_blkDevInfo.mBlkDev:
							del( missing_blkDevInfo_LIST[ missing_blkDevInfo_idx ] )
							del( invalid_blkDevInfo_LIST[ invalid_blkDevInfo_idx ] )
							invalid_blkDevInfo_idx = invalid_blkDevInfo_idx - 1
							break
						missing_blkDevInfo_idx = missing_blkDevInfo_idx + 1
					invalid_blkDevInfo_idx = invalid_blkDevInfo_idx + 1

				if len( missing_blkDevInfo_LIST ) > 0:
					missing_diskInfo = diskInfo.copy()
					missing_diskInfo.mBlkDevInfo_LIST = missing_blkDevInfo_LIST
					missing_diskInfo_DICT[ wwid ] = missing_diskInfo

				if len( invalid_blkDevInfo_LIST ) > 0:
					invalid_diskInfo = diskInfo.copy()
					invalid_diskInfo.mBlkDevInfo_LIST = invalid_blkDevInfo_LIST
					invalid_diskInfo_DICT[ wwid ] = invalid_diskInfo

		except KeyError:
			invalid_diskInfo_DICT[ wwid ] = cached_diskInfo

	for wwid in diskInfo_DICT:
		missing_diskInfo_DICT[ wwid ] = diskInfo_DICT[ wwid ]

	return ( missing_diskInfo_DICT, invalid_diskInfo_DICT )

####################################################################################################
def no_output( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "no_output" )

####################################################################################################
def disk_list( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "disk_list" )

	sorted_wwid_LIST = diskInfo_DICT.keys()
	sorted_wwid_LIST.sort()
	for wwid in sorted_wwid_LIST:
		print wwid

####################################################################################################
def disk_blkdev_list( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "disk_blkdev_list" )

	sorted_wwid_LIST = diskInfo_DICT.keys()
	sorted_wwid_LIST.sort()
	for wwid in sorted_wwid_LIST:
		diskInfo = diskInfo_DICT[ wwid ]
		print diskInfo.mWWID + '  ::',
		for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
			print blkDevInfo.mBlkDev,
		print

####################################################################################################
def disk_blkdev_diff_list( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "disk_blkdev_diff_list" )

	missing_diskInfo_DICT, invalid_diskInfo_DICT = generate_disk_blkdev_differences_LIST( vi_errorAndLog_obj )

	if len( missing_diskInfo_DICT ) > 0:
		print 'Missing:'
		for missing_wwid in missing_diskInfo_DICT:
			missing_diskInfo = missing_diskInfo_DICT[ missing_wwid ]
			print '    ' + missing_wwid
			for missing_blkDevInfo in missing_diskInfo.mBlkDevInfo_LIST:
				print '        ' + missing_blkDevInfo.mBlkDev

	if len( missing_diskInfo_DICT ) > 0 and len( invalid_diskInfo_DICT ) > 0:
		print ''

	if len( invalid_diskInfo_DICT ) > 0:
		print 'Invalid:'
		for invalid_wwid in invalid_diskInfo_DICT:
			invalid_diskInfo = invalid_diskInfo_DICT[ invalid_wwid ]
			print '    ' + invalid_wwid
			for invalid_blkDevInfo in invalid_diskInfo.mBlkDevInfo_LIST:
				print '        ' + invalid_blkDevInfo.mBlkDev

####################################################################################################
def lvm_list( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "lvm_list" )

	sorted_vgInfo_LIST = vgInfo_uuid_DICT.values()
	sorted_vgInfo_LIST.sort()

	for vgInfo in sorted_vgInfo_LIST:
		lvInfo_DICT = vgInfo.mLVinfo_name_DICT
		sorted_lv_name_LIST = lvInfo_DICT.keys()
		sorted_lv_name_LIST.sort()
		for lv_name in sorted_lv_name_LIST:
			lvInfo = lvInfo_DICT[ lv_name ]
			print "%s_%s" % ( lvInfo.mVGname, lvInfo.mName )

####################################################################################################
def disk_and_lvm_data_XML( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	output_disk_and_lvm_data_XML( vi_errorAndLog_obj, options_OBJ.indentXMLlvl, options_OBJ.skipLVM, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT )


####################################################################################################
def disk_XML( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, dummy1, dummy2, dummy3):
	"""
	This routine
	"""
#
	output_disk_XML( vi_errorAndLog_obj, options_OBJ.indentXMLlvl, diskInfo_DICT, item_LIST )

####################################################################################################
def scan_vdisk_data_XML( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	output_scan_vdisk_data_XML( vi_errorAndLog_obj, options_OBJ.indentXMLlvl, options_OBJ.doVDiskMntScan, item_LIST, diskInfo_DICT, vgInfo_uuid_DICT )

####################################################################################################
# END
