#!/usr/bin/env python
#
# Copyright (C) 2011-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Konrad Rzeszutek, Joshua Nicholas
#
"""Utilities for forking.
"""

import os
import sys
import time

__all__ = [	'vi_fork',
			'vi_close_fds' ]

def vi_fork():
	"""
	Fork a double-child.
	"""
	pid = os.fork()

	if pid != 0:
		os.waitpid(pid, 0)
		# Parent exit
		return pid

	os.setsid()
	pid = os.fork()
	if pid != 0:
		# Child #1 exit.
		os._exit(0)

	# Child #2
	os.chdir("/")
	os.umask(0)
	return pid

def vi_close_fds(except_fd_LIST):
	"""
	Close all of the file descriptors, except the ones on the passed in list.
	"""
	for fd in range(0, 1024):
		if fd not in except_fd_LIST:
			try:
				os.close(fd)
			except:
				pass

	os.open("/dev/null", os.O_RDWR)
	os.dup2(0, 1)
	os.dup2(0, 1)

