#!/usr/bin/env python
#
# Copyright (C) 2011-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for miscellaneous purposes
"""

import os
import sys
import re

import st_utils

####################################################################################################

__all__ = [	'ps_process_OBJ'
		,	'vi_get_ps_process_dict' ]

####################################################################################################

class ps_process_OBJ:

	def __init__(self):
		self.mPID				= None
		self.mParentPID			= None
		self.mSessionID			= None
		self.mState				= None
		self.mPercentCPU		= None
		self.mPercentMemory		= None
		self.mThreadCount		= None
		self.mWhichProcessor	= None
		self.mResidentSetSize	= None
		self.mVirtualMemorySize	= None
		self.mPendingSignals	= None
		self.mWaitChannel		= None
		self.mCmd_LIST			= None

	def __repr__(self):
		return "mPID: %s mParentPID: %s mSessionID: %s mState: %s mPercentCPU: %s mPercentMemory: %s mThreadCount: %s mWhichProcessor: %s mResidentSetSize: %s mVirtualMemorySize: %s mPendingSignals: %s mWaitChannel: %s mCmd: %s" % (
				self.mPID
			,	self.mParentPID
			,	self.mSessionID
			,	self.mState
			,	self.mPercentCPU
			,	self.mPercentMemory
			,	self.mThreadCount
			,	self.mWhichProcessor
			,	self.mResidentSetSize
			,	self.mVirtualMemorySize
			,	self.mPendingSignals
			,	self.mWaitChannel
			,	self.mCmd_LIST )

####################################################################################################

global__cached__ps_process_DICT		= None

####################################################################################################
def vi_get_ps_process_dict( vi_errorAndLog_obj, refresh ):
	"""
	This routine returns a dict containing process information returned by ps
	"""
#
	global global__cached__ps_process_DICT

	if not refresh and global__cached__ps_process_DICT:
		return global__cached__ps_process_DICT

	cmd_LIST	= [ 'ps', '-e', '--no-headers', '-o', 'pid,ppid,sid,state,%cpu,%mem,thcount,psr,rss,vsz,pending,wchan:24,cmd' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )

	ps_process_DICT	= {}
	for ps_line in stdout_LIST:

		ps_line_LIST						= ps_line.split()	# ASSUMPTION: only cmd (LAST!) has spaces in the column element

		new_ps_process						= ps_process_OBJ()
		new_ps_process.mPID					= ps_line_LIST[  0 ]
		new_ps_process.mParentPID			= ps_line_LIST[  1 ]
		new_ps_process.mSessionID			= ps_line_LIST[  2 ]
		new_ps_process.mState				= ps_line_LIST[  3 ]
		new_ps_process.mPercentCPU			= ps_line_LIST[  4 ]
		new_ps_process.mPercentMemory		= ps_line_LIST[  5 ]
		new_ps_process.mThreadCount			= ps_line_LIST[  6 ]
		new_ps_process.mWhichProcessor		= ps_line_LIST[  7 ]
		new_ps_process.mResidentSetSize		= ps_line_LIST[  8 ]
		new_ps_process.mVirtualMemorySize	= ps_line_LIST[  9 ]
		new_ps_process.mPendingSignals		= ps_line_LIST[ 10 ]
		new_ps_process.mWaitChannel			= ps_line_LIST[ 11 ]
		new_ps_process.mCmd_LIST			= ps_line_LIST[ 12 : ]

#		if new_ps_process.mPID == '15228':
#			vi_errorAndLog_obj.vi_print_dbg( 0, "l: '%s'" % ( ps_line ) )
#			vi_errorAndLog_obj.vi_print_dbg( 0, "L: '%s'" % ( ps_line_LIST ) )
#			vi_errorAndLog_obj.vi_print_dbg( 0, new_ps_process )
#			vi_errorAndLog_obj.vi_stop()

		new_ps_process.mPID					= int( new_ps_process.mPID )
		new_ps_process.mParentPID			= int( new_ps_process.mParentPID )
		new_ps_process.mSessionID			= int( new_ps_process.mSessionID )

		ps_process_DICT[ new_ps_process.mPID ]	= new_ps_process

	global__cached__ps_process_DICT = ps_process_DICT
	return global__cached__ps_process_DICT

####################################################################################################
# END
