#!/usr/bin/env python
#
# Copyright (C) 2011-2012 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for dealing with device-mapper
"""

import st_utils

import os
import re
import traceback

from device		import *

####################################################################################################

__all__ = [	'DeviceMapper_map_OBJ'
		,	'DeviceMapper_map_dep_OBJ'
		,	'VI_DM_GRAPH_COMPONENT_TYPE__BLOCK_DEVICE'
		,	'VI_DM_GRAPH_COMPONENT_TYPE__MULTIPATH'
		,	'VI_DM_GRAPH_COMPONENT_TYPE__UNKNOWN'
		,	'VI_DM_GRAPH_COMPONENT_TYPE__LVM'
		,	'DeviceMapper_graph_component_OBJ'
		,	'get_device_mapper_major_num'
		,	'get_dm_map_open_count'
		,	'get_devNums_to_blkDev_mapping'
		,	'OLD__get_device_mapper_maps'
		,	'get_blk_dev_to_dm_map_lookup_dict'
		,	'get_device_mapper_map_graph'
		,	'get_device_mapper_maps' ]

####################################################################################################

class DeviceMapper_map_OBJ:

	def __init__( self, theMapName, theMajorNum, theMinorNum, theUUID, theLVMusing_LIST ):
		self.mMapName   	= theMapName
		self.mMajorNum  	= theMajorNum
		self.mMinorNum  	= theMinorNum
		self.mUUID      	= theUUID
		self.mDep_LIST  	= []				# DeviceMapper_map_dep_OBJ
		self.mLVMusing_LIST	= theLVMusing_LIST	# DeviceMapper_map_OBJ

	def __repr__( self ):
		lvmUsingStr	= '['
		if self.mLVMusing_LIST != None and len(self.mLVMusing_LIST) > 0:
			endIdx_in	 = len(self.mLVMusing_LIST) - 1
			for idx in range( 0, endIdx_in ):
				lvmUsingStr	+= self.mLVMusing_LIST[idx].mMapName + ', '
			lvmUsingStr	+= self.mLVMusing_LIST[endIdx_in].mMapName
		lvmUsingStr	+= ']'
		return "n: %s #: %s:%s u: %s d: %s l: %s" % \
					(	self.mMapName
					,	self.mMajorNum
					,	self.mMinorNum
					,	self.mUUID
					,	self.mDep_LIST
					,	lvmUsingStr )

	def __cmp__( self, other ):
		if not isinstance( other, DeviceMapper_map_OBJ ):
			return -1
		return cmp( self.mMapName, other.mMapName )

class DeviceMapper_map_dep_OBJ:

	def __init__( self, theBlkDevName, theBlkDevNums ):
		self.mBlkDevName    = theBlkDevName
		self.mBlkDevNums    = theBlkDevNums

	def __repr__( self ):
		return "%s %s" % ( self.mBlkDevName, self.mBlkDevNums )

	def __cmp__( self, other ):
		if not isinstance( other, DeviceMapper_map_dep_OBJ ):
			return -1
		return cmp( self.mBlkDevName, other.mBlkDevName )

VI_DM_GRAPH_COMPONENT_TYPE__BLOCK_DEVICE	= -2
VI_DM_GRAPH_COMPONENT_TYPE__MULTIPATH		= -1
VI_DM_GRAPH_COMPONENT_TYPE__UNKNOWN			=  0
VI_DM_GRAPH_COMPONENT_TYPE__LVM				=  1

class DeviceMapper_graph_component_OBJ:

	def __init__( self, theMapName, theParentMapName, theMajorNum, theMinorNum, theUUID, theDepLvl ):
		self.mMapName	 	= theMapName
		self.mParentMapName	= theParentMapName
		self.mMajorNum		= theMajorNum
		self.mMinorNum		= theMinorNum
		self.mUUID			= theUUID
		self.mDepLvl		= theDepLvl
		self.mMapType		= VI_DM_GRAPH_COMPONENT_TYPE__UNKNOWN
		self.mBlkDevName	= "dm-%s" % ( theMinorNum )
		self.mDep_LIST	 	= []		# DeviceMapper_graph_component_OBJ

	def recursive_walk_pre( self, cbFcn, cbArgTPL, cbAccumulation=None ):
		doNotRecurse_BOOL, cbAccumulation	= cbFcn( self, cbArgTPL, cbAccumulation )
		if doNotRecurse_BOOL:
			return cbAccumulation
		for dep in self.mDep_LIST:
			dep.recursive_walk_pre( cbFcn, cbArgTPL, cbAccumulation )
		return cbAccumulation

	def recursive_walk_pst( self, cbFcn, cbArgTPL, cbAccumulation=None ):
		for dep in self.mDep_LIST:
			dep.recursive_walk_pst( cbFcn, cbArgTPL, cbAccumulation )
		cbAccumulation	= cbFcn( self, cbArgTPL, cbAccumulation )
		return cbAccumulation

	def get_walk_depth_list( self ):
		return self.recursive_walk_pst( _INTERNAL__recurse_walk_depth_list, (), [] )

	def resursive_str( self, indentLvl, dmFrmt=False ):
		if not dmFrmt:
			theStr	= "%s%s" % ( '  ' * indentLvl, str(self) )
		else:
			if self.mMapType != VI_DM_GRAPH_COMPONENT_TYPE__BLOCK_DEVICE :
				theStr	= "%s%s (%d:%d) [%s]" % (	'  ' * indentLvl
												,	self.mMapName
												,	self.mMajorNum
												,	self.mMinorNum
												,	self.mUUID )
			else:
				theStr	= "%s(%d:%d)" % (	'  ' * indentLvl
										,	self.mMajorNum
										,	self.mMinorNum )
		for dep in self.mDep_LIST:
			theStr	+= '\n' + dep.resursive_str( indentLvl+1, dmFrmt )
		return theStr

	def __repr__( self ):
		if self.mParentMapName != None:
			return "n: %s [%s] #: %s:%s u: %s l: %s t: %s d: %d" % \
						(	self.mMapName
						,	self.mParentMapName
						,	self.mMajorNum
						,	self.mMinorNum
						,	self.mUUID
						,	self.mDepLvl
						,	self.mMapType
						,	len(self.mDep_LIST) )
		else:
			return "n: %s #: %s:%s u: %s l: %s t: %s d: %d" % \
						(	self.mMapName
						,	self.mMajorNum
						,	self.mMinorNum
						,	self.mUUID
						,	self.mDepLvl
						,	self.mMapType
						,	len(self.mDep_LIST) )

	def __cmp__( self, other ):
		if not isinstance( other, DeviceMapper_graph_component_OBJ ):
			return -1
		return cmp( self.mMapName, other.mMapName )

class DeviceMapperTraversal_regex_decoder_OBJ:

	def __init__( self ):
		self.mLineSub1_cmpldRegEx	= None
		self.mLineSub2_cmpldRegEx	= None
		self.mRootMap_cmpldRegEx	= None
		self.mDepMap_cmpldRegEx		= None
		self.mUUID_cmpldRegEx		= None
		self.mBlkDevDep_cmpldRegEx	= None

####################################################################################################

global_device_mapper_major_num					= -1

global__cached__devNums_to_blkDev_DICT			= None
global__cached__dmMapGraph_DICT					= None
global__cached__blkDevParentDMmap_DICT			= None
global__cached__dm_maps_DICT					= None
global__cached__blk_dev_to_dm_map_lookup_DICT	= None

####################################################################################################
def get_device_mapper_major_num( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	global global_device_mapper_major_num

	if global_device_mapper_major_num < 0:
		global_device_mapper_major_num	= get_block_device_major_num( vi_errorAndLog_obj, 'device-mapper', False )

	return global_device_mapper_major_num

####################################################################################################
def get_dm_map_open_count( vi_errorAndLog_obj, dm_map_name ):
	"""
	Get the count of open files on the dm_map
	"""
#
	info_cmd_LIST	= [ 'dmsetup', 'info', '--columns', '--noheadings', dm_map_name ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, info_cmd_LIST )

	if len( stdout_LIST ) < 1 or len( stdout_LIST[0] ) < 1:
		return -1	# map no longer exists

	dm_map_info_LIST = stdout_LIST[0].strip().split( ':' )
	if len( dm_map_info_LIST ) != 8:
#		vi_errorAndLog_obj.vi_croak(     "unexpected dm_map_info_line '%s' %s" % ( stdout_LIST[0], dm_map_info_LIST ) )
		vi_errorAndLog_obj.vi_print_err( "unexpected dm_map_info_line '%s' %s" % ( stdout_LIST[0], dm_map_info_LIST ) )
		return 666	# Doom

	return int(dm_map_info_LIST[4])

####################################################################################################
def get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, refresh ):
	"""
	This routine retrieves the dictionary of major:minor to block device name, as so:
		8:0	-> sda
		8:16	-> sdb
		7:0	-> loop0

	It does not scan the partitions (so, /dev/sda1 is out of the picture)
	"""
#
	global global__cached__devNums_to_blkDev_DICT
	if not refresh and global__cached__devNums_to_blkDev_DICT:
		return global__cached__devNums_to_blkDev_DICT

	devNums_to_blkDev_DICT = {}

	for blkDevName in os.listdir( '/sys/block' ):
		devNums, read_failed = st_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/block/%s/dev" % ( blkDevName ) )
		if read_failed:
			continue
		devNums_to_blkDev_DICT[ devNums ] = blkDevName

	global__cached__devNums_to_blkDev_DICT = devNums_to_blkDev_DICT
	return global__cached__devNums_to_blkDev_DICT

####################################################################################################
def OLD__get_device_mapper_maps( vi_errorAndLog_obj, refresh ):
	"""
	This routine provides a dictionary of DeviceMapper_map_OBJ objects. The key value is
	the name of a non-linear device mapper map. The value is the DeviceMapper_map_OBJ object, wherein
	a list of the linear maps and a list of block devices names is contained.

	"""
#
	global global__cached__dm_maps_DICT
	if not refresh and global__cached__dm_maps_DICT:
		return global__cached__dm_maps_DICT

	dm_maps_DICT = {}

	devNums_to_blkDev_DICT = get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, refresh )

	bad_tags                = [	re.compile( '\|-' )
							,	re.compile( '\|' )
							,	re.compile( '`-' ) ]
	# Matches:
	# 3600805f3001321c03aa1d28e12a40039-dfwrhas5u132 (253:26) [LVM-LFdh47RWu2DgOCOeAJK5FxJzffQbx7uTJhGKFBluGA2zeNZrnqkhkfLxEP3gJFez]
	# 3600a0b8000370e3f0000122f47d8e432 (253:53) [mpath-3600a0b8000370e3f0000122f47d8e432]
	dm_map_cmpldRegEx       = re.compile( "^(\|?)(\S+)\s+\((\d+):(\d+)\)\s+\[(\S*)]$" )
	dm_map_uuid_cmpldRegEx  = re.compile( "^(\w+)-(.+)$" )#  6000F4B000000000002101142372c7fa-6000F4B00000000000610118804fd1b7
	dm_map_dep_cmpldRegEx   = re.compile( "^(\|?)\s*\((\d+:\d+)\)$" ) # |- (65:224)

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, 'dmsetup ls --tree -o device,ascii,notrunc,uuid' )

	if 0:
		for line in stdout_LIST:
			for bad_tag in bad_tags:
				line = bad_tag.sub( '|', line )
			line = line.strip()
			vi_errorAndLog_obj.vi_print_dbg( 0, line )
		vi_stop()

	if 0:
		prevDbgLvl	= vi_errorAndLog_obj.get_debugLvl()
		vi_errorAndLog_obj.set_debugLvl( 10 )

	stdout_ITR		= iter(stdout_LIST)
	lvmUsing_LIST	= []
	orphanedLVM_LIST= []
	dm_map			= None
	try:
		line = stdout_ITR.next()
		for bad_tag in bad_tags:
			line = bad_tag.sub( '|', line )
		line = line.strip()
		while True:			# Search for the next map

			vi_errorAndLog_obj.vi_print_dbg( 5, "F: %s" % ( line ) )
			dm_map_match = dm_map_cmpldRegEx.match( line )
			if not dm_map_match:
				line = stdout_ITR.next()
				for bad_tag in bad_tags:
					line = bad_tag.sub( '', line )
				line = line.strip()
				vi_errorAndLog_obj.vi_print_dbg( 5, "F_I: %s " % (line))
				continue

			vi_errorAndLog_obj.vi_print_dbg( 4, "G: %s :: %s" % ( line, dm_map_match.group(1) ) )
			vi_errorAndLog_obj.vi_print_dbg( 5, "U: %s" % ( dm_map_match.group(4) ) )

			# Check if it is LVM
			dm_map_uuid_match = dm_map_uuid_cmpldRegEx.match( dm_map_match.group(5) )
			if dm_map_uuid_match:
				if dm_map_uuid_match.group(1) == 'LVM':
					vi_errorAndLog_obj.vi_print_dbg( 6, "Collecting LVM dm_map: %s" % ( dm_map_match.group(2) ) )
					dm_map = DeviceMapper_map_OBJ(	dm_map_match.group(2)
												,	dm_map_match.group(3)
												,	dm_map_match.group(4)
												,	dm_map_match.group(5)
												,	None )
					# Check to see if this LVM map is dependent on a non-multipath block device
					line = stdout_ITR.next()
					for bad_tag in bad_tags:
						line = bad_tag.sub( '|', line )
					line = line.strip()
					vi_errorAndLog_obj.vi_print_dbg( 5, "LVM_I: %s" % ( line ) )
					# This ingested next line should look like:
					# "'-3600805f3001321c02f8962adb414003c (253:8) [mpath-3600805f3001321c02f8962adb414003c]"
					# OR a linear map (AOE disk)
					# "'-3600805f3001321c02f8962adb414003c (253:8) []"
					# But it could look like:
					# "-- (8:6)"
					dm_map_dep_match = dm_map_dep_cmpldRegEx.match( line )
					if dm_map_dep_match:
						# yup. The next line is '- (8:6) instead of a multipath name...
						devNums = dm_map_dep_match.group(2)
						devNums_to_blkDev_DICT = get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, True )
						blkDevName = devNums_to_blkDev_DICT.get( devNums, None )
						if blkDevName:
							dm_map_dep = DeviceMapper_map_dep_OBJ( blkDevName , devNums )
							dm_map.mDep_LIST.append( dm_map_dep )
							vi_errorAndLog_obj.vi_print_dbg( 3, "Orphaned_Y: %s" % ( dm_map_dep ) )
						orphanedLVM_LIST.append( dm_map )
						vi_errorAndLog_obj.vi_print_dbg( 5, "Orphaned_I: %s: " % ( dm_map_match.group(2)))
					else:
						lvmUsing_LIST.append( dm_map )	# This LVM map is dependent on a multipath or linear map
					dm_map = None
					continue

			dm_map = DeviceMapper_map_OBJ(	dm_map_match.group(2)
										,	dm_map_match.group(3)
										,	dm_map_match.group(4)
										,	dm_map_match.group(5)
										,	lvmUsing_LIST )

			lvmUsing_LIST	= []	# Reset the lvmUsing_LIST

			vi_errorAndLog_obj.vi_print_dbg( 3, "M: %s" % ( dm_map_match.group(2) ) )

			while True:		# Find direct dependents or switch back to
				# This will read in "|- (65:224)"
				line = stdout_ITR.next()
				for bad_tag in bad_tags:
					line = bad_tag.sub( '|', line )
				line = line.strip()
				vi_errorAndLog_obj.vi_print_dbg( 5, "M_I: %s" % ( line ) )
				dm_map_dep_match = dm_map_dep_cmpldRegEx.match( line )
				if not dm_map_dep_match:
#					vi_errorAndLog_obj.vi_print_dbg( 5, "Could not match '%s' to '%s'" % ( line, dm_map_dep_cmpldRegEx.pattern ) )
					break	# Go back to looking for the next map
				# (xx:yy) -> sdX
				devNums = dm_map_dep_match.group(2)
				blkDevName = devNums_to_blkDev_DICT.get( devNums, None )
				if not blkDevName:
					devNums_to_blkDev_DICT = get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, True )	# Refresh the cache
					blkDevName = devNums_to_blkDev_DICT.get( devNums, None )
					if not blkDevName:
						# This can happen when the iSCSI session has been disconnected 
						# AND its block devices have been removed.
						vi_errorAndLog_obj.vi_print_dbg( 3, "blkDevName not found for devNums %s" % ( devNums ) )
						continue
				dm_map_dep = DeviceMapper_map_dep_OBJ(	blkDevName , devNums )
				dm_map.mDep_LIST.append( dm_map_dep )
				vi_errorAndLog_obj.vi_print_dbg( 3, "Y: %s" % ( dm_map_dep ) )

			# Add map to the dict OR update the existing map with any new LVM maps
			cur_dm_map	= dm_maps_DICT.get( dm_map.mMapName, None )
			if not cur_dm_map:
				dm_maps_DICT[ dm_map.mMapName ] = dm_map
				vi_errorAndLog_obj.vi_print_dbg( 2, "Added dm_map: %s" % ( dm_map ) )
			else:
				for lvm_dm_map in dm_map.mLVMusing_LIST:
					for cur_lvm_dm_map in cur_dm_map.mLVMusing_LIST:
						if cur_lvm_dm_map.mMapName != lvm_dm_map.mMapName:
							continue
						break
					else:
						vi_errorAndLog_obj.vi_print_dbg( 6, "Adding lvm_dm_map: '%s' %s" % ( dm_map.mMapName, lvm_dm_map ) )
						cur_dm_map.mLVMusing_LIST.append( lvm_dm_map )
				vi_errorAndLog_obj.vi_print_dbg( 2, "Updated cur_dm_map: %s" % ( cur_dm_map ) )
			dm_map = None

	except StopIteration:	# done
		if dm_map:
			# Add map to the dict OR update the existing map with any new LVM maps
			cur_dm_map	= dm_maps_DICT.get( dm_map.mMapName, None )
			if not cur_dm_map:
				dm_maps_DICT[ dm_map.mMapName ] = dm_map
				vi_errorAndLog_obj.vi_print_dbg( 2, "Added dm_map: %s" % ( dm_map ) )
			else:
				for lvm_dm_map in dm_map.mLVMusing_LIST:
					for cur_lvm_dm_map in cur_dm_map.mLVMusing_LIST:
						if cur_lvm_dm_map.mMapName != lvm_dm_map.mMapName:
							continue
						break
					else:
						vi_errorAndLog_obj.vi_print_dbg( 6, "Adding lvm_dm_map: '%s' %s" % ( dm_map.mMapName, lvm_dm_map ) )
						cur_dm_map.mLVMusing_LIST.append( lvm_dm_map )
				vi_errorAndLog_obj.vi_print_dbg( 2, "Updated cur_dm_map: %s" % ( cur_dm_map ) )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '\nDUMP:' )
		for dm_map_name, dm_map in dm_maps_DICT.iteritems():
			vi_errorAndLog_obj.vi_print_dbg( 0, "n: %s    #: %s:%s u: %s\n    d: %s" % \
					(	dm_map.mMapName
					,	dm_map.mMajorNum
					,	dm_map.mMinorNum
					,	dm_map.mUUID
					,	dm_map.mDep_LIST ) )
			for lvm_dm_map in dm_map.mLVMusing_LIST:
				vi_errorAndLog_obj.vi_print_dbg( 0, "    l: n: %s    #: %s:%s u: %s" % \
						(	lvm_dm_map.mMapName
						,	lvm_dm_map.mMajorNum
						,	lvm_dm_map.mMinorNum
						,	lvm_dm_map.mUUID ) )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
#		vi_errorAndLog_obj.vi_stop()
#		VERIFY:
#		dmsetup ls --tree -o ascii,notrunc,uuid,nodevice | sed -e 's/[[:space:]]*|[[:space:]]*`-//' -e 's/[[:space:]]*`-//' | grep -v ' \[LVM' | sort | uniq

	if vi_errorAndLog_obj.vi_should_print_dbg( 3 ) and len(orphanedLVM_LIST):
		vi_errorAndLog_obj.vi_print_dbg( 0, "Found a LVMs that are NOT under multipath and hence cannot be disconnected." )
		for lvm in orphanedLVM_LIST:
			vi_errorAndLog_obj.vi_print_dbg( 0, "    %s (%s)" % ( lvm.mMapName , lvm.mUUID) )

	if 0:
		vi_errorAndLog_obj.set_debugLvl( prevDbgLvl )

	global__cached__dm_maps_DICT = dm_maps_DICT
	return global__cached__dm_maps_DICT

####################################################################################################
def get_blk_dev_to_dm_map_lookup_dict( vi_errorAndLog_obj, refresh ):
	"""
	This routine retrieves the dictionary of block devices names mapped to their
	device mapper maps. For example:
		sdc -> 1494554000000000000000000020000000100000002000000
		sdd -> 1494554000000000000000000020000000100000002000000
		sda -> SMaxtor_3200_2CAHXASE
	
	and so on.
	"""
#
	global global__cached__blk_dev_to_dm_map_lookup_DICT
	if not refresh and global__cached__blk_dev_to_dm_map_lookup_DICT:
		return global__cached__blk_dev_to_dm_map_lookup_DICT

	blk_dev_to_dm_map_lookup_DICT	= {}

	dm_maps_DICT	= get_device_mapper_maps( vi_errorAndLog_obj, refresh )
	for dm_map_name, dm_map in dm_maps_DICT.iteritems():
		for dm_map_dep in dm_map.mDep_LIST:
			blk_dev_to_dm_map_lookup_DICT[ dm_map_dep.mBlkDevName ]	= dm_map_name

	global__cached__blk_dev_to_dm_map_lookup_DICT = blk_dev_to_dm_map_lookup_DICT
	return global__cached__blk_dev_to_dm_map_lookup_DICT

####################################################################################################
def _INTERNAL__get_next_line( vi_errorAndLog_obj, dmTravrsDecoder, stdout_ITR ):
	"""
	This routine
	"""
#
	try:
		while True:
			nextLine	= stdout_ITR.next().rstrip()
			if len(nextLine) > 1:
				break
	except:
		return None

	nextLine	= dmTravrsDecoder.mLineSub1_cmpldRegEx.sub( ' `-', nextLine )
	nextLine	= dmTravrsDecoder.mLineSub2_cmpldRegEx.sub( '   ', nextLine )

#	if nextLine.startswith( 'foobarDEVICEmapperMAP' ):
#		vi_errorAndLog_obj.set_debugLvl( 10 )
#	if nextLine.startswith( 'foobarDEVICEmapperMAP' ):
#		vi_errorAndLog_obj.set_debugLvl( 0 )

#	if nextLine.startswith( 'foobarDEVICEmapperMAP' ):
#		vi_errorAndLog_obj.vi_stop()
#		return None

	return nextLine

####################################################################################################
def _INTERNAL__set_map_type( vi_errorAndLog_obj, dmTravrsDecoder, theDMgrphComp ):
	"""
	This routine
	"""
#
	if len( theDMgrphComp.mUUID ) < 1:
		return

	uuid_match	= dmTravrsDecoder.mUUID_cmpldRegEx.match( theDMgrphComp.mUUID )
	if uuid_match:
		if uuid_match.group(1) == 'LVM':
			theDMgrphComp.mMapType	= VI_DM_GRAPH_COMPONENT_TYPE__LVM
		if uuid_match.group(1) == 'mpath':
			theDMgrphComp.mMapType	= VI_DM_GRAPH_COMPONENT_TYPE__MULTIPATH

####################################################################################################
def _INTERNAL__recursivly_add_map_deps( vi_errorAndLog_obj, dmTravrsDecoder, devNums_to_blkDev_DICT, dmMapGraph_DICT, blkDevParentDMmap_DICT, theRootMap, theDMgrphComp, stdout_ITR, depLvl, curIndentLen ):
	"""
	This routine
	"""
#
	nextLine	= None
	while True:

		if nextLine == None:
			nextLine	= _INTERNAL__get_next_line( vi_errorAndLog_obj, dmTravrsDecoder, stdout_ITR )
			if nextLine == None:
				return None

		line		= nextLine
		nextLine	= None

		vi_errorAndLog_obj.vi_print_dbg( 5, "Nl: [%d] %s" % ( depLvl, line ) )

		####### Is it a root level map
		rootMap_match = dmTravrsDecoder.mRootMap_cmpldRegEx.match( line )
		if rootMap_match:

			if depLvl > 0:
				return line

			rootDMmap	= DeviceMapper_graph_component_OBJ(	rootMap_match.group(1)
														,	None
														,	int(rootMap_match.group(2))
														,	int(rootMap_match.group(3))
														,	rootMap_match.group(4)
														,	0
														)

			_INTERNAL__set_map_type( vi_errorAndLog_obj, dmTravrsDecoder, rootDMmap )

			vi_errorAndLog_obj.vi_print_dbg( 4, "Rm: [0] %s" % ( rootDMmap ) )

			dmMapGraph_DICT[ rootDMmap.mMapName ]	= rootDMmap

			theRootMap								= rootDMmap
			theDMgrphComp							= rootDMmap
			continue

		####### Is it a sub level dependency map
		depMap_match = dmTravrsDecoder.mDepMap_cmpldRegEx.match( line )
		if depMap_match:

			foundIndentLen	= len(depMap_match.group(1))
			if foundIndentLen > curIndentLen:
				newDepLvl	= 	depLvl+1
			else:
				return line	# Not for this level :: pop

			depDMgrphComp	= DeviceMapper_graph_component_OBJ(	depMap_match.group(2)
															,	theRootMap.mMapName
															,	int(depMap_match.group(3))
															,	int(depMap_match.group(4))
															,	depMap_match.group(5)
															,	newDepLvl
															)

			_INTERNAL__set_map_type( vi_errorAndLog_obj, dmTravrsDecoder, depDMgrphComp )

			vi_errorAndLog_obj.vi_print_dbg( 4, "Dm: [%d] %s" % ( depDMgrphComp.mDepLvl, depDMgrphComp ) )

			theDMgrphComp.mDep_LIST.append( depDMgrphComp )

			nextLine	= _INTERNAL__recursivly_add_map_deps(	vi_errorAndLog_obj
																,	dmTravrsDecoder
																,	devNums_to_blkDev_DICT
																,	dmMapGraph_DICT
																,	blkDevParentDMmap_DICT
																,	theRootMap
																,	depDMgrphComp
																,	stdout_ITR
																,	depDMgrphComp.mDepLvl
																,	foundIndentLen
																)

			continue

		####### Is it a block device dependency
		blkDevDep_match = dmTravrsDecoder.mBlkDevDep_cmpldRegEx.match( line )
		if blkDevDep_match:

			foundIndentLen	= len(blkDevDep_match.group(1))

			devNums			= "%s:%s" % ( blkDevDep_match.group(2), blkDevDep_match.group(3) )

			blkDevName = devNums_to_blkDev_DICT.get( devNums, None )
			if not blkDevName:
				devNums_to_blkDev_DICT	= get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, True )
				blkDevName				= devNums_to_blkDev_DICT.get( devNums, None )
			if not blkDevName:
				vi_errorAndLog_obj.vi_print_dbg( 2, "orphaned LVM component [%d] %s : %s : %s" % ( depLvl, theRootMap.mMapName, theDMgrphComp.mMapName, line ) )
				continue

			depDMgrphComp			= DeviceMapper_graph_component_OBJ(	blkDevName
																	,	theDMgrphComp.mMapName
																	,	int(blkDevDep_match.group(2))
																	,	int(blkDevDep_match.group(3))
																	,	None
																	,	depLvl+1
																	)
			depDMgrphComp.mMapType		= VI_DM_GRAPH_COMPONENT_TYPE__BLOCK_DEVICE
			depDMgrphComp.mBlkDevName	= blkDevName

			theDMgrphComp.mDep_LIST.append( depDMgrphComp )

			vi_errorAndLog_obj.vi_print_dbg( 4, "Bd: [%d] %s" % ( depDMgrphComp.mDepLvl, depDMgrphComp ) )

			dm_map		= blkDevParentDMmap_DICT.get( theDMgrphComp.mMapName, None )
			if not dm_map:
				dm_map	= DeviceMapper_map_OBJ(	theDMgrphComp.mMapName
											,	theDMgrphComp.mMajorNum
											,	theDMgrphComp.mMinorNum
											,	theDMgrphComp.mUUID
											,	[]
											)
				blkDevParentDMmap_DICT[ dm_map.mMapName ]	= dm_map

			dm_map_dep	= DeviceMapper_map_dep_OBJ( blkDevName , devNums )
			dm_map.mDep_LIST.append( dm_map_dep )

			if theDMgrphComp.mParentMapName:
				lvmDMmap	= DeviceMapper_map_OBJ(	theRootMap.mMapName
												,	theRootMap.mMajorNum
												,	theRootMap.mMinorNum
												,	theRootMap.mUUID
												,	[]
												)
				dm_map.mLVMusing_LIST.append( lvmDMmap )

			continue

		if line != "No devices found":
			vi_errorAndLog_obj.vi_print_err( "unexpected line [%d] '%s'" % ( depLvl, line ) )

####################################################################################################
def get_device_mapper_map_graph( vi_errorAndLog_obj, refresh ):
	"""
	This routine parses dmsetup ls --tree information and builds a cooresponding graph of
	the various maps.
	"""
#
	global global__cached__dmMapGraph_DICT
	global global__cached__blkDevParentDMmap_DICT
	if not refresh and global__cached__dmMapGraph_DICT:
		return global__cached__dmMapGraph_DICT, global__cached__blkDevParentDMmap_DICT

	#
	# mRootMap and mDepMap matches:
	#	3600805f3001321c03aa1d28e12a40039-dfwrhas5u132 (253:26) [LVM-LFdh47RWu2DgOCOeAJK5FxJzffQbx7uTJhGKFBluGA2zeNZrnqkhkfLxEP3gJFez]
	#	3600a0b8000370e3f0000122f47d8e432 (253:53) [mpath-3600a0b8000370e3f0000122f47d8e432]
	#
	# mUUID matches:
	#	LVM-LFdh47RWu2DgOCOeAJK5FxJzffQbx7uTJhGKFBluGA2zeNZrnqkhkfLxEP3gJFez
	#	mpath-3600a0b8000370e3f0000122f47d8e432
	#
	# mBlkDevDep matches:
	#	(8:32)
	#
	dmTravrsDecoder							= DeviceMapperTraversal_regex_decoder_OBJ()
	dmTravrsDecoder.mLineSub1_cmpldRegEx	= re.compile( " \|-" )
	dmTravrsDecoder.mLineSub2_cmpldRegEx	= re.compile( " \| " )
	dmTravrsDecoder.mRootMap_cmpldRegEx		= re.compile( "^(\S+)\s+\((\d+):(\d+)\)\s+\[(\S*)]$" )
	dmTravrsDecoder.mDepMap_cmpldRegEx		= re.compile( "^(\s+)`-(\S+)\s+\((\d+):(\d+)\)\s+\[(\S*)]$" )
	dmTravrsDecoder.mUUID_cmpldRegEx		= re.compile( "^(\w+)-(.+)$" )
	dmTravrsDecoder.mBlkDevDep_cmpldRegEx	= re.compile( "^(\s+)`- \((\d+):(\d+)\)$" )

	cmdList					= [ 'dmsetup', 'ls', '--tree', '-o', 'device,ascii,notrunc,uuid' ]
	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, cmdList )

	devNums_to_blkDev_DICT	= get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, refresh )

	if 0:
		stdout_LIST = \
"""6000F4B00BAAD0880011000110dead01-6000F4B00BAAD0660041000beef00060 (253:77) [LVM-PIs57WvEjg3nqxWUvP1ZoPwnQYWdMqkgTLhrECZisEs5k7ZeZRKJy6Db0Q1SywGz]
 `-3600a0b800036e48a0000421d48e361b1 (253:66) [mpath-3600a0b800036e48a0000421d48e361b1]
    |- (65:64)
    `- (65:128)""".split( '\n' )

	if 0:
		for idx, line in enumerate(stdout_LIST):
			vi_errorAndLog_obj.vi_print_dbg( 0, "%2d] %s" % ( idx, line.rstrip() ) )

	if 0:
		prevDbgLvl	= vi_errorAndLog_obj.get_debugLvl()
		vi_errorAndLog_obj.set_debugLvl( 10 )
	else:
		prevDbgLvl	= None

	blkDevParentDMmap_DICT	= {}
	dmMapGraph_DICT			= {}
	stdout_ITR				= iter(stdout_LIST)

	_INTERNAL__recursivly_add_map_deps(	vi_errorAndLog_obj
									,	dmTravrsDecoder
									,	devNums_to_blkDev_DICT
									,	dmMapGraph_DICT
									,	blkDevParentDMmap_DICT
									,	None
									,	None
									,	stdout_ITR
									,	0
									,	0
									)

	for blkDevParentDMmapName, blkDevParentDMmap in blkDevParentDMmap_DICT.iteritems():
		st_utils.vi_sort_uniq( blkDevParentDMmap.mDep_LIST, False )
		st_utils.vi_sort_uniq( blkDevParentDMmap.mLVMusing_LIST, False )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '\nDeviceMapperGraph:\n' )
		for rootDMmapName, rootDMmap in dmMapGraph_DICT.iteritems():
			vi_errorAndLog_obj.vi_print_dbg( 0, rootDMmap.resursive_str(0) )
		vi_errorAndLog_obj.vi_print_dbg( 0, '\nBlkDevParentDMmaps:\n' )
		for blkDevParentDMmapName, blkDevParentDMmap in blkDevParentDMmap_DICT.iteritems():
			vi_errorAndLog_obj.vi_print_dbg( 0, blkDevParentDMmap )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
#		vi_errorAndLog_obj.vi_stop()

	if prevDbgLvl != None:
		vi_errorAndLog_obj.set_debugLvl( prevDbgLvl )

	global__cached__dmMapGraph_DICT			= dmMapGraph_DICT
	global__cached__blkDevParentDMmap_DICT	= blkDevParentDMmap_DICT
	return global__cached__dmMapGraph_DICT, global__cached__blkDevParentDMmap_DICT

####################################################################################################
def get_device_mapper_maps( vi_errorAndLog_obj, refresh ):
	"""
	This routine traverses the graph list generated by get_device_mapper_map_graph() and
	extracts simplified device mapper map information.
	"""
#
	global global__cached__dm_maps_DICT
	if not refresh and global__cached__dm_maps_DICT:
		return global__cached__dm_maps_DICT

	dmMapGraph_LIST, blkDevParentDMmap_DICT	= get_device_mapper_map_graph( vi_errorAndLog_obj, refresh )

	global__cached__dm_maps_DICT	= blkDevParentDMmap_DICT
	return global__cached__dm_maps_DICT

####################################################################################################
def _INTERNAL__recurse_walk_depth_list( dmGraphComp, cbArgTPL, cbAccumulation ):
	"""
	This routine
	"""
#
	walkDepth_LIST_LIST	= cbAccumulation

	while dmGraphComp.mDepLvl+1 <= len(walkDepth_LIST_LIST):
		walkDepth_LIST_LIST.append( [] )

	found	= False
	for knownDMgraphComp in walkDepth_LIST_LIST[ dmGraphComp.mDepLvl ]:
		if knownDMgraphComp.mMapName == dmGraphComp.mMapName:
			found	= True
			break
	if not found:
		walkDepth_LIST_LIST[ dmGraphComp.mDepLvl ].append( dmGraphComp )

	return walkDepth_LIST_LIST

####################################################################################################
# END
